<?php

namespace WcMipConnector\Service;

defined('ABSPATH') || exit;

use WcMipConnector\Client\Base\Exception\ClientErrorException;
use WcMipConnector\Client\BigBuy\Model\ShippingOption;
use WcMipConnector\Client\BigBuy\Model\ShippingRequest;
use WcMipConnector\Client\BigBuy\Shipping\Service\CarrierService;
use WcMipConnector\Client\BigBuy\Shipping\Service\ShippingService as BigBuyShippingService;
use WcMipConnector\Enum\StatusTypes;
use WcMipConnector\Factory\BigBuy\ShippingFactory;
use WcMipConnector\Manager\ConfigurationOptionManager;
use WcMipConnector\Manager\ShippingServiceManager;

class ShippingService
{
    private const TAX_DIVISION = 100.0;
    private const ROUND_PRECISION = 2;
    private const MIN_DATE_TO_UPDATE_CARRIERS = 7;

    /** @var TaxesService */
    protected $taxesService;
    /** @var BigBuyShippingService */
    private $shippingServiceBigBuy;
    /** @var PublicationOptionsService */
    private $publicationOptionsService;
    /** @var LoggerService  */
    private $loggerService;
    /** @var CarrierService */
    private $carrierService;
    /** @var ShippingServiceManager */
    private $shippingServiceManager;

    /**
     * ShippingService constructor.
     * @throws \Exception
     */
    public function __construct()
    {
        $this->taxesService = new TaxesService();
        $apiKey = ConfigurationOptionManager::getApiKey();
        $this->shippingServiceBigBuy = BigBuyShippingService::getInstance($apiKey);
        $this->carrierService = CarrierService::getInstance($apiKey);
        $this->publicationOptionsService = new PublicationOptionsService();
        $this->shippingServiceManager = new ShippingServiceManager();
        $this->loggerService = new LoggerService();
    }

    /**
     * @param string $countryIsoCode
     * @param string $postCode
     * @param array $products
     * @return array
     */
    public function getPackageShippingCost(string $countryIsoCode, string $postCode, array $products): array
    {
        $cheapestCarrier = [];
        $shippingFactory = new ShippingFactory();
        $requestParams = $shippingFactory->create($countryIsoCode, $postCode, $products);
        $carriers = $this->getShippingCosts($requestParams);

        if (!\is_array($carriers)) {
            return [];
        }

        $shippingServices = $this->shippingServiceManager->getDisabledNamesIndexedByName();

        $conversionFactor = $this->publicationOptionsService->getConversionFactor();
        $taxRate = 0.0;
        $taxId = 0;

        try {
            $tax = $this->taxesService->getTaxWithMaxRate($countryIsoCode);
            $taxRate = $tax['Rate'] / self::TAX_DIVISION;
            $taxId = $tax['Id'];
        } catch (\Exception $exception) {
            $this->loggerService->info('Empty taxes for country with iso_code '.$countryIsoCode);
        }

        foreach ($carriers as $shippingCost) {
            if (\array_key_exists($shippingCost->shippingService->name, $shippingServices)) {
                continue;
            }

            $shippingCostWithConversionRate = $shippingCost->cost * $conversionFactor;
            $shippingTax = $shippingCostWithConversionRate * $taxRate;
            $shippingWithTaxesAndConversionRate = \round($shippingCostWithConversionRate + $shippingTax, self::ROUND_PRECISION);

            if (
                empty($cheapestCarrier)
                || !\array_key_exists('carrier_cost', $cheapestCarrier)
                || $shippingWithTaxesAndConversionRate < $cheapestCarrier['carrier_cost']
            ) {
                $cheapestCarrier['carrier_name'] = $shippingCost->shippingService->serviceName;
                $cheapestCarrier['carrier_delay'] = $shippingCost->shippingService->delay;
                $cheapestCarrier['carrier_cost'] = $shippingWithTaxesAndConversionRate;
                $cheapestCarrier['taxes'] = \round($shippingTax, self::ROUND_PRECISION);
                $cheapestCarrier['id'] = $taxId;
            }
        }

        return $cheapestCarrier;
    }

    /**
     * @param ShippingRequest $requestParams
     * @return ShippingOption[]
     */
    public function getShippingCosts(ShippingRequest $requestParams): array
    {
        $cacheId = CacheService::generateCacheKey(__METHOD__, $requestParams);

        if (CacheService::getInstance()->has($cacheId)) {
            $cacheResult = CacheService::getInstance()->get($cacheId);

            if (\is_array($cacheResult)) {
                return $cacheResult;
            }
        }

        $shippingOptionsResponse = [];

        try {
            $shippingOptionsResponse = $this->shippingServiceBigBuy->getShippingCost($requestParams);
        } catch (ClientErrorException $exception) {
            if ($exception->getCode() === StatusTypes::HTTP_TOO_MANY_REQUESTS) {
                return [];
            }
        }

        CacheService::getInstance()->set($cacheId, $shippingOptionsResponse, 600);

        return $shippingOptionsResponse;
    }

    /**
     * @return array
     */
    public function getShippingServices(): array
    {
        $cacheId = CacheService::generateCacheKey(__METHOD__);
        if (CacheService::getInstance()->has($cacheId)) {
            return CacheService::getInstance()->get($cacheId);
        }

        $carriers = $this->carrierService->getCarriers() ?: [];

        CacheService::getInstance()->set($cacheId, $carriers, 10);

        return $carriers;
    }

    public function updateShippingServices(): void
    {
        $shippingServices = $this->getShippingServices();

        if (!$shippingServices || !$this->shouldUpdateCarriers()) {
            return;
        }

        $shippingServicesIndexById = $this->shippingServiceManager->getAllIndexedById();

        foreach ($shippingServices as $shippingService) {
            if (\array_key_exists($shippingService['id'], $shippingServicesIndexById)) {
                $this->shippingServiceManager->update($shippingService);

                continue;
            }

            $this->shippingServiceManager->insert($shippingService);
        }
    }

    private function shouldUpdateCarriers(): bool
    {
        $lastApiCarrierUpdateDate = ConfigurationOptionManager::getLastCarrierUpdate();

        $currentDateTime = new \DateTime('now');
        $currentTime = $currentDateTime->format('Y-m-d H:i:s');

        if (!$lastApiCarrierUpdateDate) {
            ConfigurationOptionManager::setLastCarrierUpdate($currentTime);

            return true;
        }

        $lastApiCarrierUpdateDateTime = \DateTime::createFromFormat('Y-m-d H:i:s', $lastApiCarrierUpdateDate);
        $intervalFromLastUpdate = $currentDateTime->diff($lastApiCarrierUpdateDateTime);

        if ($intervalFromLastUpdate->days < self::MIN_DATE_TO_UPDATE_CARRIERS) {
            return false;
        }

        ConfigurationOptionManager::setLastCarrierUpdate($currentTime);

        return true;
    }
}