<?php

namespace WcMipConnector\Service;

use WcMipConnector\Client\Base\Exception\ClientErrorException;
use WcMipConnector\Client\MIP\Customer\Service\StockService as StockMipService;
use WcMipConnector\Entity\WCProduct;
use WcMipConnector\Exception\WooCommerceApiExceptionInterface;
use WcMipConnector\Factory\StockFactory;
use WcMipConnector\Manager\ConfigurationOptionManager;
use WcMipConnector\Manager\ProductManager;
use WcMipConnector\Manager\ProductMapManager;
use WcMipConnector\Manager\VariationManager;
use WcMipConnector\Manager\VariationMapManager;

defined('ABSPATH') || exit;

class StockService
{
    private const BATCH_VALUE_TO_QUERY = 5000;

    /** @var ProductMapManager */
    protected $productMapManager;

    /** @var VariationMapManager */
    protected $variationMapManager;

    /** @var StockFactory */
    protected $stockFactory;

    /** @var ProductManager */
    protected $productManager;

    /** @var SystemService */
    protected $systemService;

    /** @var VariationManager */
    protected $variationManager;

    /** @var LoggerService */
    protected $logger;

    /** @var VariationService */
    protected $variationService;

    public function __construct()
    {
        $this->productMapManager = new ProductMapManager();
        $this->variationMapManager = new VariationMapManager();
        $this->stockFactory = new StockFactory();
        $this->productManager = new ProductManager();
        $this->systemService = new SystemService();
        $this->variationManager = new VariationManager();
        $this->logger = new LoggerService();
        $this->variationService = new VariationService();
    }

    /**
     * @return bool
     */
    public function updateStock(): bool
    {
        $lastStockUpdateDate = ConfigurationOptionManager::getLastStockUpdate();

        $currentTime = date('Y-m-d H:i:s');
        if (!$lastStockUpdateDate) {
            ConfigurationOptionManager::setLastStockUpdate($currentTime);
            return false;
        }

        mt_srand(strtotime($lastStockUpdateDate));
        $hoursRandom = mt_rand(22, 24);
        $minutesRandom = mt_rand(0, 59);
        $secondsRandom = mt_rand(1, 15);

        sleep($secondsRandom);

        $nextCatalogStockUpdate = strtotime(
            ' +'.$hoursRandom.' hours +'. $minutesRandom . ' minutes',
            strtotime($lastStockUpdateDate)
        );

        if ($nextCatalogStockUpdate > strtotime($currentTime)) {
            return true;
        }

        ConfigurationOptionManager::setLastStockUpdate($currentTime);
        $accessToken = ConfigurationOptionManager::getAccessToken();

        try {
            $this->logger->debug($currentTime.' Retrieving product stocks information to update');
            $productStockResponse = StockMipService::getInstance($accessToken)->getStocks();
        } catch(ClientErrorException $exception) {
            $this->logger->critical($exception->getMessage());

            return false;
        }

        if (empty($productStockResponse)) {
            return false;
        }

        $productStockIndexedById = [];
        $productVariationStockAndProductIdIndexedById = [];
        $productStockToUpdate = [];
        $variationStockToUpdate = [];
        $productIdsIndexedById = [];

        foreach ($productStockResponse as $productStock) {
            if (!$productStock->productActive || !$productStock->productId) {
                continue;
            }

            $productIdsIndexedById[$productStock->productId] = $productStock->productId;

            if ($productStock->productVariationId === "0") {
                $productStockIndexedById[$productStock->productId] = $productStock->stockQuantity;
                continue;
            }

            $productVariationStockAndProductIdIndexedById[$productStock->productVariationId]['stock'] = $productStock->stockQuantity;
            $productVariationStockAndProductIdIndexedById[$productStock->productVariationId]['productId'] = $productStock->productId;
        }

        if (!empty($productIdsIndexedById)) {
            $productMapIdsIndexedByProductMapId = $this->productMapManager->findByProductMapIdsIndexedByProductMapId(array_keys($productIdsIndexedById));

            foreach ($productStockIndexedById as $productId => $productStockQuantity) {
                if (!array_key_exists($productId, $productMapIdsIndexedByProductMapId)) {
                    continue;
                }

                $productShopId = $productMapIdsIndexedByProductMapId[$productId];
                $productStockToUpdate[$productShopId] = $productShopId;
            }
        }

        if (!empty($productStockToUpdate)) {
            try {
                $this->processProductsByQuery($productStockToUpdate);
            } catch (WooCommerceApiExceptionInterface $wooCommerceApiConnectionException) {
                $this->logger->error('Updating product stock - ERROR: '. $wooCommerceApiConnectionException->getMessage());
            }
        }

        if (!empty($productVariationStockAndProductIdIndexedById)) {
            $variationMapsIndexedByVariationMapId = $this->variationMapManager->findByVariationMapIdsIndexedByVariationMapId(array_keys($productVariationStockAndProductIdIndexedById));

            foreach ($productVariationStockAndProductIdIndexedById as $variationId => $variation) {
                if (!array_key_exists($variationId, $variationMapsIndexedByVariationMapId)) {
                    continue;
                }

                $variationShopId = $variationMapsIndexedByVariationMapId[$variationId];
                $variationStockToUpdate[$variationShopId] = $variationShopId;
            }
        }

        if (!empty($variationStockToUpdate)) {
            $this->processVariationsByQuery($variationStockToUpdate);
        }

        return true;
    }

    /**
     * @param array $productStockToUpdate
     * @throws WooCommerceApiExceptionInterface
     */
    public function processProductsByBatch(array $productStockToUpdate): void
    {
        $batches = array_chunk($productStockToUpdate, $this->systemService->getBatchValue(), true);

        foreach ($batches as $batchToProcess) {
            $this->productManager->updateCollection($batchToProcess);
        }
    }

    /**
     * @param array $productStockToUpdate
     * @throws WooCommerceApiExceptionInterface
     */
    public function processProductsByQuery(array $productStockToUpdate): void
    {
        $batches = \array_chunk($productStockToUpdate, self::BATCH_VALUE_TO_QUERY, true);

        foreach ($batches as $batchToProcess) {
            $this->productManager->updateProductStockByProductIds(\array_values($batchToProcess));
        }
    }

    /**
     * @param array $variationStockToUpdate
     */
    public function processVariationsByQuery(array $variationStockToUpdate): void
    {
        $batches = \array_chunk($variationStockToUpdate, self::BATCH_VALUE_TO_QUERY, true);

        foreach ($batches as $batchToProcess) {
            $this->productManager->updateProductStockByProductIds(\array_values($batchToProcess));
        }
    }

    /**
     * @param array $variationStockToUpdate
     * @param int $productParentId
     */
    public function processVariationsByBatch(array $variationStockToUpdate, int $productParentId): void
    {
        $variationsBatchResponse = $this->variationManager->updateCollection($variationStockToUpdate, $productParentId);
        $this->checkVariationBatchErrors($variationStockToUpdate, $variationsBatchResponse);
    }

    /**
     * @param array $variationIndexedByVariationId
     * @param array $variationsBatchResponse
     */
    private function checkVariationBatchErrors(array $variationIndexedByVariationId, array $variationsBatchResponse): void
    {
        if (!$variationsBatchResponse) {
            return;
        }

        $variationMapIndex = 0;

        /** @var WCProduct $variationMap */
        foreach ($variationIndexedByVariationId as $variationId => $variationMap) {
            if (!\array_key_exists($variationMapIndex, $variationsBatchResponse)) {
                break;
            }

            $variation = $variationsBatchResponse[$variationMapIndex];
            $variationMapIndex++;

            if (!$variation) {
                continue;
            }

            if (\array_key_exists('error', $variation) && $variation['error'] && $variation['error']['code']) {
                $setProductError = '';

                if ($this->variationService->isInvalidVariationIdErrorCode($variation['error']['code'])) {
                    $this->productManager->deleteBySku($variationMap->sku);
                    $setProductError = ' - Invalid SKU deleted, this variation will be processed in another file';
                }

                $this->logger->getInstance()->error('VariationShopId: '.$variationId.' has been not processed in stock request. Batch response: '
                    .$variation['error']['code']. $setProductError, $variation['error']);
            }
        }
    }
}