<?php

namespace WcMipConnector\Service;

defined('ABSPATH') || exit;

use WcMipConnector\Exception\NoResultException;
use WcMipConnector\Exception\WooCommerceApiExceptionInterface;
use WcMipConnector\Factory\MethodFactory;
use WcMipConnector\Factory\ZoneFactory;
use WcMipConnector\Manager\ConfigurationOptionManager;
use WcMipConnector\Manager\ProductManager;
use WcMipConnector\Manager\ZoneManager;

class ZoneService
{
    private const METHOD_POSITION = 0;

    /** @var ZoneManager */
    protected $zoneManager;
    /** @var ZoneFactory */
    protected $zoneFactory;
    /** @var MethodFactory */
    protected $methodFactory;
    /** @var ZoneFactory */
    protected $productManager;
    /** @var ShippingService */
    protected $shippingService;
    /** @var LoggerService */
    protected $logger;

    public function __construct()
    {
        $this->zoneManager = new ZoneManager();
        $this->zoneFactory = new ZoneFactory();
        $this->methodFactory = new MethodFactory();
        $this->productManager = new ProductManager();
        $this->shippingService = new ShippingService();
        $loggerService = new LoggerService();
        $this->logger = $loggerService->getInstance();
    }

    /**
     * @return array
     * @throws WooCommerceApiExceptionInterface
     */
    public function findAllZones(): array
    {
        $findAllZones = $this->zoneManager->findAllZones();
        $zonesIndexedById = \array_column($findAllZones, 'id', 'name');
        if (!array_key_exists(ZoneFactory::ALL_WORLD_ZONE, $zonesIndexedById)) {
            $zoneModel = $this->zoneFactory->create();
            $zones = $this->zoneManager->createZones(\json_decode(\json_encode($zoneModel), true));

            return $this->findShippingZoneByZoneId($zones['id']);
        }

        return $this->findShippingZoneByZoneId($zonesIndexedById[ZoneFactory::ALL_WORLD_ZONE]);
    }

    /**
     * @param int $zoneId
     *
     * @return array
     * @throws WooCommerceApiExceptionInterface
     */
    public function findShippingZoneByZoneId(int $zoneId): array
    {
        $listCarriers = $this->zoneManager->findShippingZoneByZoneId($zoneId);
        $carriers = \array_column($listCarriers, 'instance_id', 'method_id');
        if (!array_key_exists(ZoneFactory::LOCAL_PICKUP, $carriers)) {
            $methodModel = $this->methodFactory->setLocalPickupMethod();
            $listCarriers = $this->zoneManager->createShippingZones($zoneId, \json_decode(\json_encode($methodModel), true));
        }

        return $listCarriers;
    }

    /**
     * @param array $shippingRates
     * @return array
     */
    public function shippingMethodsAvailable(array $shippingRates): array
    {
        $shippingMethods = [];

        $bigBuyCarrierOption = ConfigurationOptionManager::getCarrierOption();
        foreach ($shippingRates as $shippingRateId => $shippingRate) {
            if ($bigBuyCarrierOption && ZoneFactory::LOCAL_PICKUP === $shippingRate->method_id) {
                continue;
            }

            $shippingMethods[$shippingRateId] = $shippingRate;
        }

        return $shippingMethods;
    }

    /**
     * @param string $label
     * @param \WC_Shipping_Rate $method
     * @return string
     * @throws WooCommerceApiExceptionInterface
     */
    public function modifyShippingLabel(string $label, \WC_Shipping_Rate $method): string
    {
        $methodSanitize = strtok($method->id, ':');

        if ($methodSanitize !== ZoneFactory::LOCAL_PICKUP) {
            return $label;
        }

        $shippingServices = $this->getCarriersResponse();

        if (!$shippingServices) {
            return $label;
        }

        if (\array_key_exists('carrier_delay', $shippingServices)
            && array_key_exists('carrier_cost', $shippingServices)
            && array_key_exists('carrier_name', $shippingServices)) {
            $method->set_label($shippingServices['carrier_name'].' ' .$shippingServices['carrier_delay'].' - '
                .str_replace('.', ',', $shippingServices['carrier_cost']).' '.get_woocommerce_currency_symbol());

            return $method->get_label();
        }

        return $label;
    }

    /**
     * @param \WC_Shipping_Rate $method
     * @throws WooCommerceApiExceptionInterface
     */
    public function findDataToProcess(\WC_Shipping_Rate $method): void
    {
        if (!is_checkout()) {
            return;
        }

        $methodSanitize = strtok($method->id, ':');

        if ($methodSanitize !== ZoneFactory::LOCAL_PICKUP) {
            return;
        }

        $shippingServices = $this->getCarriersResponse();

        if (!$shippingServices) {
            return;
        }
    }

    /**
     * @return array
     * @throws WooCommerceApiExceptionInterface
     */
    public function getCarriersResponse(): array
    {
        $products = [];

        if (empty(WC()->shipping()->packages)) {
            return [];
        }

        $productsCart = WC()->cart->cart_contents;

        foreach ($productsCart as $productCart) {
            $product['sku'] = $productCart['data']->get_sku();
            $product['quantity'] = $productCart['quantity'];
            $products[] = $product;
        }
        
        $countryIsoCode = WC()->shipping()->packages[0]['destination']['country'];
        $postalCode = WC()->shipping()->packages[0]['destination']['postcode'];

        return $this->shippingService->getPackageShippingCost($countryIsoCode, $postalCode, $products);
    }

    /**
     * @param \WC_Cart $cart
     *
     * @return \WC_Cart
     * @throws WooCommerceApiExceptionInterface
     */
    public function getTotalCart(\WC_Cart $cart): \WC_Cart
    {
        $method = WC()->session->get( 'chosen_shipping_methods' );
        $methodSanitize = strtok($method[self::METHOD_POSITION], ':');

        if ($methodSanitize !== ZoneFactory::LOCAL_PICKUP) {
            return $cart;
        }

        $shippingServices = $this->getCarriersResponse();

        if (!$shippingServices) {
            return $cart;
        }

        if (
            array_key_exists('carrier_delay', $shippingServices)
            && array_key_exists('carrier_cost', $shippingServices)
            && array_key_exists('taxes', $shippingServices)
            && array_key_exists('id', $shippingServices)
        ) {
            /** calculate_shipping indirectly calls findDataToProcess through woocommerce_after_shipping_rate hook */
            /** @var \WC_Shipping_Rate[] $shippingMethods */
            $shippingTotalWithoutTaxes = $shippingServices['carrier_cost'] - $shippingServices['taxes'];
            $shippingMethods = $cart->calculate_shipping();
            $shippingMethod = $shippingMethods[0];
            $shippingMethod->set_label($shippingServices['carrier_name'].' '.$shippingServices['carrier_delay']);
            $shippingMethod->set_cost($shippingTotalWithoutTaxes);
            $shippingMethod->set_taxes([$shippingServices['id'] => $shippingServices['taxes']]);
            $cart->set_shipping_total($shippingTotalWithoutTaxes);
            $cart->set_shipping_tax($shippingServices['taxes']);
            $cart->set_shipping_taxes([$shippingServices['id'] => $shippingServices['taxes']]);
            $totalTax = $cart->get_subtotal_tax() + $shippingServices['taxes'];
            $cart->set_total_tax($totalTax);
            $total = $cart->get_subtotal() + $totalTax + $shippingTotalWithoutTaxes - $cart->get_discount_total() - $cart->get_discount_tax();
            $cart->set_total($total);

            return $cart;
        }

        /** calculate_shipping indirectly calls findDataToProcess through woocommerce_after_shipping_rate hook */
        /** @var \WC_Shipping_Rate[] $shippingMethods */
        $shippingMethod = $cart->calculate_shipping();
        $shippingMethod[0]->set_label($shippingServices['carrier_name'].' '.$shippingServices['carrier_delay']);
        $shippingMethod[0]->set_cost($shippingServices['carrier_cost']);
        $shippingMethod[0]->set_taxes([0 => 0]);
        $total = $cart->get_subtotal() + $shippingServices['carrier_cost'] - $cart->get_discount_total() - $cart->get_discount_tax();
        $cart->set_total($total);
        $subTotalTax = $cart->get_subtotal_tax();
        $cart->set_total_tax($subTotalTax);

        return $cart;
    }

    public function createAllWorldZoneIfNotExists(): void
    {
        $allWorldZoneId = null;

        try {
            $allWorldZoneId = $this->getAllWorldZoneId();
        } catch (NoResultException $exception) {
            $zoneModel = $this->zoneFactory->create();
            try {
                $zones = $this->zoneManager->createZones(\json_decode(\json_encode($zoneModel), true));
                $allWorldZoneId = $zones['id'];
            } catch (WooCommerceApiExceptionInterface $exception) {
                $this->logger->error('Creating All World Zone - Zone Model: '.\json_encode($zoneModel).' - Error message: '.$exception->getMessage());
            }
        }

        if (!$allWorldZoneId) {
            return;
        }

        try {
            $this->enableLocalPickup($allWorldZoneId);
        } catch (NoResultException $exception) {
            $methodModel = $this->methodFactory->setLocalPickupMethod();
            try {
                $this->zoneManager->createShippingZones($allWorldZoneId, \json_decode(\json_encode($methodModel), true));
            } catch (WooCommerceApiExceptionInterface $exception) {
                $this->logger->error('Creating Zone Method - Method Model: '.\json_encode($methodModel).' - Error message: '.$exception->getMessage());
            }
        }
    }

    /**
     * @return int
     * @throws NoResultException
     */
    private function getAllWorldZoneId(): int
    {
        $zonesIndexedById = null;

        try {
            $findAllZones = $this->zoneManager->findAllZones();
            $zonesIndexedById = \array_column($findAllZones, 'id', 'name');
        } catch (WooCommerceApiExceptionInterface $exception) {
            $this->logger->error('Exception error int getAllWorldZoneId: '. $exception->getMessage());
        }

        if (!array_key_exists(ZoneFactory::ALL_WORLD_ZONE, $zonesIndexedById)) {
            throw new NoResultException('No ID found.');
        }

        return $zonesIndexedById[ZoneFactory::ALL_WORLD_ZONE];
    }

    /**
     * @param int $zoneId
     * @return int
     * @throws WooCommerceApiExceptionInterface
     * @throws NoResultException
     */
    private function getLocalPickupId(int $zoneId): int
    {
        $carriers = [];
        $shippingZone = $this->zoneManager->findShippingZoneByZoneId($zoneId);

        foreach ($shippingZone as $shippingMethod) {
            if ($shippingMethod['enabled'] && $shippingMethod['method_id'] === ZoneFactory::LOCAL_PICKUP) {
                $carriers = \array_column($shippingZone, 'instance_id', 'method_id');
                break;
            }
        }

        if (!array_key_exists(ZoneFactory::LOCAL_PICKUP, $carriers)) {
            throw new NoResultException('No ID found');
        }

        return $carriers[ZoneFactory::LOCAL_PICKUP];
    }

    /**
     * @return bool
     */
    public function checkIfAllWorldZoneLocalPickupExists(): bool
    {
        try {
            $allZoneWorldId = $this->getAllWorldZoneId();
        } catch (NoResultException $e) {
            return false;
        }

        try {
            $this->getLocalPickupId($allZoneWorldId);
        } catch (NoResultException $e) {
            return false;
        }

        return true;
    }

    /**
     * @param int $zoneId
     * @return mixed
     * @throws NoResultException
     */
    private function enableLocalPickup(int $zoneId)
    {
        $shippingZoneMethodsIndexedById = [];
        $shippingZoneDisabledMethodsIndexedById = [];

        try {
            $shippingZone = $this->zoneManager->findShippingZoneByZoneId($zoneId);
        } catch (WooCommerceApiExceptionInterface $exception) {
            $this->logger->error('ShippingZone cannot be reached - Error message: '.$exception->getMessage());
            throw new NoResultException('ShippingZone cannot be reached - Error message: '.$exception->getMessage());
        }

        foreach ($shippingZone as $shippingMethod) {
            if (!isset($shippingMethod['enabled'], $shippingMethod['method_id']) || $shippingMethod['method_id'] !== ZoneFactory::LOCAL_PICKUP) {
                continue;
            }

            $shippingZoneIndexedById = \array_column($shippingZone, 'instance_id', 'method_id');

            if ($shippingMethod['enabled']) {
                $shippingZoneMethodsIndexedById = $shippingZoneIndexedById;
                break;
            }
            $shippingZoneDisabledMethodsIndexedById = $shippingZoneIndexedById;
        }

        if (\array_key_exists(ZoneFactory::LOCAL_PICKUP, $shippingZoneMethodsIndexedById)) {
            return $shippingZoneMethodsIndexedById[ZoneFactory::LOCAL_PICKUP];
        }

        if (!array_key_exists(ZoneFactory::LOCAL_PICKUP, $shippingZoneDisabledMethodsIndexedById)) {
            throw new NoResultException('LocalPickup not found at shippingZoneMethods or shippingZoneDisabledMethods');
        }

        try {
            $localPickUp = $shippingZoneDisabledMethodsIndexedById[ZoneFactory::LOCAL_PICKUP];
            $this->zoneManager->enableShippingZoneMethodByZoneIdAndMethodId($zoneId, $localPickUp);

            return $localPickUp;
        } catch (WooCommerceApiExceptionInterface $exception) {
            $this->logger->error('Enabling shipping zone method id: '.$localPickUp.' - Error message: '.$exception->getMessage());
            throw new NoResultException('Enabling shipping zone method id: '.$localPickUp.' - Error message: '.$exception->getMessage());
        }
    }
}